__productname__ = "analytix"
__version__ = "0.1.0a1"
__description__ = "analytix is a simple yet powerful API wrapper to make getting analytical information from your favourite services easier than ever."
__url__ = "https://github.com/parafoxia/analytix"
__docs__ = "https://analytix.readthedocs.io/en/latest/"
__author__ = "Ethan Henderson"
__license__ = "BSD-3-Clause"

from .errors import *
